import os, sys, math, random, time, pygame
from pygame.locals import *
from pygame.compat import geterror

import const, util, item, bonus, paddle, var

class Button():
    def __init__(self, filename, x, y, w):
        self.name = filename
        self.image = util.load_image("images/gui/" + filename + ".png")
        self.image_hi = util.load_image("images/gui/" + filename + "_hi.png")
        self.rect = self.image.get_rect()
        self.rect[0] = x + (w - self.rect[2])/2
        self.rect[1] = y
        self.hover = False
        
    def draw(self):
        if self.hover:
            var.game.screen.blit(self.image_hi, self.rect)
        else:
            var.game.screen.blit(self.image, self.rect)            

    def clicked(self, pos):
        return self.rect.collidepoint(pos)

    def mouseover(self, pos):
    #changes self.hover, returns True if changed, False if not changed    
        if self.rect.collidepoint(pos):
            if self.hover:   #already highlighted
                return False
            else:   #highlight on
                self.hover = True
                return True
        else:      
            if self.hover:  #highlight off
                self.hover = False
                return True
            else:  #already off
                return False
        
class MessageBox():
    def __init__(self, game, name):
        self.game = game
        self.border_color = pygame.Color(0xe0e0e0e0)
        self.border_height = 2
        self.image = util.load_image("images/gui/messagebox.png", -2)
        self.contents = self.image.get_rect()
        self.contents[0] = (const.SCREEN_WIDTH - self.contents[2]) / 2
        self.contents[1] = (const.SCREEN_HEIGHT - const.SCREEN_Y0 - self.contents[3]) / 2 + const.SCREEN_Y0 
        self.name = name
        self.title = util.load_image("images/gui/" + name + ".png", -2)
        self.textpos = self.title.get_rect()
        self.textpos[0] = self.contents[0] + (self.contents[2] - self.textpos[2])/2
        self.textpos[1] = self.contents[1] + 40
        self.buttons = []
        self.active = False
        self.dirty = False

    def draw(self):
        if not self.dirty:
            return
        self.dirty = False
        self.game.screen.blit(self.image, self.contents)
        self.game.screen.blit(self.title, self.textpos)
        for b in self.buttons:
            self.buttons[b].draw()

    def display(self):
        for b in self.buttons:        
            self.buttons[b].hover = False
        self.active = True
        self.dirty = True
        self.game.cur_message = self.name

    def hide(self):
        self.active = False
        self.game.cur_message = ''

    def mouse_move(self, pos):
        for b in self.buttons:
            if self.buttons[b].mouseover(pos):
                self.dirty = True
                break

    def event_loop(self):
        self.draw()
        for event in pygame.event.get():
            if event.type == QUIT:
                return False
            elif event.type == KEYDOWN:
                if event.key == K_RETURN:
                    self.hide()
            elif event.type == MOUSEBUTTONDOWN:
                self.mouse_click(event.pos)
            elif event.type == MOUSEMOTION:
                self.mouse_move(event.pos)
        return True
      
class LevelUp(MessageBox):
    def __init__(self, game):
        MessageBox.__init__(self, game, "levelup")
        self.buttons = {"ok" : Button("continue", self.contents[0], self.contents[1] + 140, self.contents[2]),
                        "main" : Button("mainmenu", self.contents[0], self.contents[1] + 220, self.contents[2])}
        
    def hide(self):
        self.active = False
        self.game.cur_message = ''
        self.game.level_up()

    def mouse_click(self, pos):
        if self.buttons["ok"].clicked(pos):
            self.hide()
            return
        elif self.buttons["main"].clicked(pos):
            self.active = False
            self.game.messages["main"].display()

class MainMenu(MessageBox):
    def __init__(self, game):
        MessageBox.__init__(self, game, "main")
        self.buttons = {"ok" : Button("newgame", self.contents[0], self.contents[1] + 140, self.contents[2]),
                        "exit" : Button("exit", self.contents[0], self.contents[1] + 220, self.contents[2])}

    def display(self):
        MessageBox.display(self)
        self.game.screen.blit(self.game.background, (0, const.SCREEN_Y0))   #erase spiders and stuff
        pygame.display.flip()
        
    def hide(self):
        self.active = False
        self.game.cur_message = ''
        self.game.restart()

    def mouse_click(self, pos):
        if self.buttons["ok"].clicked(pos):
            self.hide()
            return
        elif self.buttons["exit"].clicked(pos):
            pygame.quit()
            sys.exit()

class GameOver(MessageBox):
    def __init__(self, game, success):
        if success:
            MessageBox.__init__(self, game, "youwin")
        else:
            MessageBox.__init__(self, game, "gameover")            

        self.buttons = {"main" : Button("mainmenu", self.contents[0], self.contents[1] + 140, self.contents[2])}

    def mouse_click(self, pos):
        if self.buttons["main"].clicked(pos):
            self.active = False
            self.game.messages["main"].display()

class HUD(pygame.sprite.DirtySprite):
    def __init__(self, game):
        pygame.sprite.DirtySprite.__init__(self)
        self.game = game
        self.image = util.load_image("images/gui/wall.png")
        self.rect = self.image.get_rect()
        self.lives = self.game.font.render("Lives: " + str(self.game.lives), True, pygame.Color("black"))
        self.score = self.game.font.render("Score: 0", True, pygame.Color("black"))
        self.level = self.game.font.render("Level: 0", True, pygame.Color("black"))
        self.bonus = self.game.font.render("Bonuses:", True, pygame.Color("black"))
        self.combo = self.game.font.render("Combo: 0", True, pygame.Color("black"))
        self.DIAL_FRAMES = 51
        self.image_bonus = {}
        self.frame_bonus = {}
        self.r_bonus = 8   #same as $r in util/make_dial.pl
        for name in game.bonusdata:    #dials for bonus timing
            image = util.load_image("images/gui/dial_" + name + ".png", -2)
            self.image_bonus[name] = [ image.subsurface(2+(self.r_bonus*2+5)*i, 0, self.r_bonus*2+1, self.r_bonus*2+1) for i in range(self.DIAL_FRAMES+1) ]
            self.frame_bonus[name] = 0

    def update(self):
        self.game.screen.blit(self.image, (0, 0))
        self.game.screen.blit(self.lives, (10, 15))
        self.game.screen.blit(self.score, (150, 15))
        self.game.screen.blit(self.level, (300, 15))
        self.game.screen.blit(self.bonus, (450, 15))
        x = 560
        y = 8
        self.game.screen.blit(self.image_bonus["net"][self.frame_bonus["net"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["wide"][self.frame_bonus["wide"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["fire"][self.frame_bonus["fire"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["freeze"][self.frame_bonus["freeze"]], (x, y))

        y += self.r_bonus*2+4
        x = 560
        self.game.screen.blit(self.image_bonus["sticky"][self.frame_bonus["sticky"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["strong"][self.frame_bonus["strong"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["missile"][self.frame_bonus["missile"]], (x, y))
        x += self.r_bonus*2+4
        self.game.screen.blit(self.image_bonus["bomb"][self.frame_bonus["bomb"]], (x, y))
        self.game.screen.blit(self.combo, (650, 15))

    def update_all(self):
        self.update_score()
        self.update_lives()
        self.update_combo()
        self.update_level()
    
    def update_score(self):
        score_str = str(self.game.score)
        self.score = self.game.font.render("Score: " + score_str, True, pygame.Color("black"))
        self.update()

    def update_lives(self):
        lives_str = str(self.game.lives)
        self.lives = self.game.font.render("Lives: " + lives_str, True, pygame.Color("black"))
        self.update()

    def update_combo(self):
        self.combo = self.game.font.render("Combo: " + str(self.game.combo), True, pygame.Color("black"))
        self.update()

    def update_level(self):
        self.level = self.game.font.render("Level: " + str(self.game.level), True, pygame.Color("black"))
        self.update()

    def update_bonus(self, name, p):
        if p == 0:
            self.frame_bonus[name] = 0
        else:
            self.frame_bonus[name] = (p-1)*(1-self.DIAL_FRAMES)/(100-1) + self.DIAL_FRAMES
        
    def reinit(self):
        for name in self.frame_bonus:
            self.frame_bonus[name] = 0
            
