import os, sys, math, random, time, pygame
from pygame.locals import *
from pygame.compat import geterror

import const, util, image, item, bonus, explosion

class Paddle(pygame.sprite.DirtySprite):
    def __init__(self, game):
        pygame.sprite.DirtySprite.__init__(self)
        self.game = game
        self.data = {   #represents all combinations of paddle states
            "-1,0":[util.load_image("images/paddleBlu_short.png", -2), 0],
            "0,0":[util.load_image("images/paddleBlu.png", -2), 0],
            "1,0":[util.load_image("images/paddleBlu_wide.png", -2), 0],
            "-1,1":[util.load_image("images/paddleRed_short.png", -2), 0],
            "0,1":[util.load_image("images/paddleRed.png", -2), 0],
            "1,1":[util.load_image("images/paddleRed_wide.png", -2), 0],
            }

        for i in range(-1,2):   #save width/2 for each image
            key = str(i)+','
            self.data[key+'0'][1] = self.data[key+'1'][1] = self.data[key+'0'][0].get_width()/2;

        self.image_dead = util.load_image("images/paddleBlack.png", -2)
        self.image = self.data["0,0"][0]
        self.rect = self.image.get_rect()
        self.width = self.rect[2] / 2
        self.height = self.rect[3] / 2
        self.friction = 0.5
        self.accel = 0.1
        self.speed = 5
        self.strong = 0
        self.dead = False
        self.wide = 0
        self.init()
        
    def init(self):
        self.dead = False
        self.expand(0)
        self.strengthen(False)
        self.vel = 0
        self.pos = const.SCREEN_WIDTH / 2   #center
        self.rect[0] = self.pos - self.width
        self.rect[1] = const.PADDLE_Y - self.height
        self.wide = 0

    def expand(self, use):
        if use == 1:
            if self.wide < 1:
                self.wide += 1
        elif use == -1:
            if self.wide > -1:
                self.wide -= 1
        else:
            self.wide = 0
        key = str(self.wide) + ',' + str(self.strong)
        self.image = self.data[key][0]
        self.width = self.data[key][1]
        self.rect[2] = self.width * 2;

    def strengthen(self, use):
        if use == True:
            if self.strong < 1:
                self.strong += 1
        else:
            if self.strong > 0:
                self.strong -= 1
        key = str(self.wide) + ',' + str(self.strong)
        self.image = self.data[key][0]                
        
    def update(self):
        self.dirty = 1
        if self.pos <= self.width:
            self.pos = self.width
        elif self.pos >= const.SCREEN_WIDTH - self.width:
            self.pos = const.SCREEN_WIDTH - self.width
        self.rect[0] = self.pos - self.width
        if self.dead:
            return
        for brick in list(self.game.bricks):
            if brick.killer and pygame.sprite.collide_mask(brick, self):
                brick.die()
                self.die_maybe()
                self.game.update_score(1)
                self.game.update_combo()
                e = explosion.Explosion(brick.rect, True, False)
                break
            elif brick.shrinker and pygame.sprite.collide_mask(brick, self):
                brick.die()
                self.expand(-1)
                break

    def die_maybe(self):
        if self.game.bonusdata["strong"]["cnt"] > 0:
            return
        self.image = self.image_dead
        self.width = self.data["0,0"][1]
        self.wide = 0
        self.dead = True

    def key_down(self, key):
        if key == pygame.K_LEFT:
            #self.vel -= self.accel
            self.vel = -self.speed
            self.pos = self.pos + self.vel            
        elif key == pygame.K_RIGHT:
            #self.vel += self.accel
            self.vel = self.speed
            self.pos = self.pos + self.vel
        self.update()

    def key_up(self, key):
        if key == pygame.K_LEFT or key == pygame.K_RIGHT:
            self.vel = 0
        self.update()

    def mouse_move(self, pos):
        self.vel = pos[0] - self.pos
        self.pos = pos[0]
        self.update()
        
